@@ -464,9 +464,11 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
                  'auth_method': discovery_auth_method,
                  'username': discovery_auth_username,
                  'password': discovery_auth_password},
+                'iscsiadm -m node --op show -p %s' % location,
                 'iscsiadm -m discoverydb -t sendtargets -I %(iface)s'
                 ' -p %(location)s --discover' % {'iface': interface,
-                                                 'location': location}]
+                                                 'location': location},
+                'iscsiadm -m node --op show -p %s' % location]
             self.assertEqual(expected_cmds, self.cmds)
             # Reset to run with a different transport type
             self.cmds = list()
@@ -509,8 +511,10 @@ class ISCSIConnectorTestCase(test_connector.ConnectorTestCase):
         expected_cmds = [
             'iscsiadm -m discoverydb -t sendtargets -p %s -I default'
             ' --op new' % location,
+            'iscsiadm -m node --op show -p %s' % location,
             'iscsiadm -m discoverydb -t sendtargets -I default -p %s'
-            ' --discover' % location]
+            ' --discover' % location,
+            'iscsiadm -m node --op show -p %s' % location]
         self.assertEqual(expected_cmds, self.cmds)
 
         self.assertRaises(exception.TargetPortalNotFound,
@@ -1497,3 +1501,91 @@ Setting up iSCSI targets: unused
             mock.call('/dev/disk/by-id/scsi-wwn'),
             mock.call('/dev/disk/by-id/dm-...'),
             mock.call('/dev/disk/by-id/scsi-...')])
+
+    @mock.patch.object(iscsi.ISCSIConnector, '_run_iscsiadm_bare')
+    def test_get_node_startup_values(self, run_iscsiadm_bare_mock):
+        name1 = 'volume-00000001-1'
+        name2 = 'volume-00000001-2'
+        name3 = 'volume-00000001-3'
+        vol = {'id': 1, 'name': name1}
+        location = '10.0.2.15:3260'
+        iqn1 = 'iqn.2010-10.org.openstack:%s' % name1
+        iqn2 = 'iqn.2010-10.org.openstack:%s' % name2
+        iqn3 = 'iqn.2010-10.org.openstack:%s' % name3
+        connection_properties = self.iscsi_connection(vol, [location], [iqn1])
+
+        node_startup1 = "manual"
+        node_startup2 = "automatic"
+        node_startup3 = "manual"
+        node_values = (
+            '# BEGIN RECORD 2.0-873\n'
+            'node.name = %s\n'
+            'node.tpgt = 1\n'
+            'node.startup = %s\n'
+            'iface.hwaddress = <empty>\n'
+            '# END RECORD\n'
+            '# BEGIN RECORD 2.0-873\n'
+            'node.name = %s\n'
+            'node.tpgt = 1\n'
+            'node.startup = %s\n'
+            'iface.hwaddress = <empty>\n'
+            '# END RECORD\n'
+            '# BEGIN RECORD 2.0-873\n'
+            'node.name = %s\n'
+            'node.tpgt = 1\n'
+            'node.startup = %s\n'
+            'iface.hwaddress = <empty>\n'
+            '# END RECORD\n') % (iqn1, node_startup1, iqn2, node_startup2,
+                                 iqn3, node_startup3)
+        run_iscsiadm_bare_mock.return_value = (node_values, None)
+
+        node_startups =\
+            self.connector._get_node_startup_values(
+                connection_properties['data'])
+        expected_node_startups = {iqn1: node_startup1, iqn2: node_startup2,
+                                  iqn3: node_startup3}
+        self.assertEqual(node_startups, expected_node_startups)
+
+    @mock.patch.object(iscsi.ISCSIConnector, '_execute')
+    def test_get_node_startup_values_no_nodes(self, exec_mock):
+        connection_properties = {'target_portal': 'ip1:port1'}
+        no_nodes_output = ''
+        no_nodes_err = 'iscsiadm: No records found\n'
+        exec_mock.return_value = (no_nodes_output, no_nodes_err)
+        res = self.connector._get_node_startup_values(connection_properties)
+        self.assertEqual({}, res)
+        exec_mock.assert_called_once_with(
+            'iscsiadm', '-m', 'node', '--op', 'show', '-p',
+            connection_properties['target_portal'],
+            root_helper=self.connector._root_helper, run_as_root=True,
+            check_exit_code=(0, 21))
+
+    @mock.patch.object(iscsi.ISCSIConnector, '_get_node_startup_values')
+    @mock.patch.object(iscsi.ISCSIConnector, '_iscsiadm_update')
+    def test_recover_node_startup_values(self, iscsiadm_update_mock,
+                                         get_node_startup_values_mock):
+        name1 = 'volume-00000001-1'
+        name2 = 'volume-00000001-2'
+        name3 = 'volume-00000001-3'
+        vol = {'id': 1, 'name': name1}
+        location = '10.0.2.15:3260'
+        iqn1 = 'iqn.2010-10.org.openstack:%s' % name1
+        iqn2 = 'iqn.2010-10.org.openstack:%s' % name2
+        iqn3 = 'iqn.2010-10.org.openstack:%s' % name3
+        connection_properties = self.iscsi_connection(vol, [location], [iqn1])
+        recover_connection = self.iscsi_connection(vol, [location], [iqn2])
+
+        node_startup1 = "manual"
+        node_startup2 = "automatic"
+        node_startup3 = "manual"
+        get_node_startup_values_mock.return_value = {iqn1: node_startup1,
+                                                     iqn2: node_startup2,
+                                                     iqn3: node_startup3}
+
+        old_node_startup_values = {iqn1: node_startup1,
+                                   iqn2: "manual",
+                                   iqn3: node_startup3}
+        self.connector._recover_node_startup_values(
+            connection_properties['data'], old_node_startup_values)
+        iscsiadm_update_mock.assert_called_once_with(
+            recover_connection['data'], "node.startup", "manual")
